/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

package fr.greyc.pantheon.ariane.ui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;

import com.sun.management.UnixOperatingSystemMXBean;

import fr.greyc.pantheon.ariane.Ariane;

import java.lang.management.ManagementFactory;
import java.lang.management.OperatingSystemMXBean;
import java.text.DecimalFormat;

/**
 * A <tt>AboutBox</tt> displays general information about
 * Ariane such as authors, version number, memory statistics, etc.
 *
 * @author Régis CLOUARD- Oct 01, 1998
 * @version Sébastien LEFÈVRE, Richard LOBRÉAU - Feb 21, 1999
 * @version Adrien SCHRÈQUE, Hicham AMADI - Nov 05, 2000
 * @version Mathieu JOETS - Sep 09, 2001
 * @version Régis CLOUARD - Oct 16, 2001
 * @version Régis CLOUARD - Dec 21, 2006 (look&feel revision)
 */
public final class AboutBox extends JDialog {
	/**
	 * Creates the "about" window and displays program information.
	 * 
	 * @param parent the frame that modal box is attached to.
	 */
	public AboutBox( JFrame parent ) {
		super(parent, Ariane.lang.getString("AboutBox.title"), true);
		JTabbedPane tabbedPane = new JTabbedPane();

		// Tab 1: Logo
		{
			JPanel panel = new JPanel();
			Box panel1=Box.createVerticalBox();
			panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.lang.getString("AboutBox.version.number")));
			panel1.add(new JLabel(Ariane.VERSION_ID, JLabel.CENTER));
			panel1.add(Box.createVerticalStrut(10));
			panel1.add(new JLabel(new ImageIcon(getClass().getClassLoader().getResource(Ariane.iconDir+"Splash.jpg"))), BorderLayout.CENTER);
			panel1.add(Box.createVerticalStrut(10));
			panel1.add(Box.createGlue());
			panel.add(panel1);
			tabbedPane.addTab(Ariane.lang.getString("AboutBox.version.title"), panel);
		}
		getContentPane().add(tabbedPane);
		pack();
		setResizable(false);

		// Tab 2: version - authors
		{
			Box panel=Box.createVerticalBox();

			JPanel panel1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
			panel.add( panel1);
			panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.lang.getString("AboutBox.warning")));
			String text = Ariane.lang.getString("AboutBox.license.text");
			JTextArea t = new JTextArea(text);
			Font font = new Font("Serif",Font.PLAIN,13);
			t.setFont(font);
			t.setEditable(false);
			panel1.add(t);
			tabbedPane.addTab(Ariane.lang.getString("AboutBox.license.title"), panel);
		}

		// Tab 3 authors
		{
			Box panel=Box.createVerticalBox();
			JPanel panel1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
			panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.lang.getString("AboutBox.contribution")));
			Box box1 = Box.createVerticalBox();
			for (int i=0; i< Ariane.AUTHORS.length;i++)
				box1.add(new JLabel(" - "+Ariane.AUTHORS[i]));
			panel1.add(box1);
			panel.add(panel1);

			JPanel panel2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
			panel2.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.lang.getString("AboutBox.address")));
			Box box2 = Box.createVerticalBox();
			box2.add(new JLabel("GREYC - Image team"));
			box2.add(new JLabel("UMR 6072 CNRS - ENSICAEN - University of Caen"));
			box2.add(new JLabel("6, boulevard du maréchal Juin"));
			box2.add(new JLabel("F-14050 Caen cedex (France)"));
			box2.add(new JLabel("https://www.greyc.fr/node/36"));
			panel2.add(box2);
			panel.add(panel2);
			tabbedPane.addTab(Ariane.lang.getString("AboutBox.authors.title"), panel);
		}

		// Tab 4 statistics
		{
			Box panel=Box.createVerticalBox();
			System.gc();
			JPanel statics;
			DecimalFormat df = new DecimalFormat("#.##");
			double stat;

			statics = new JPanel(new FlowLayout(FlowLayout.LEFT));
			statics.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.lang.getString("AboutBox.statistics.usedMemory")));
			stat = Runtime.getRuntime().totalMemory()/(1024.0*1024.0);
			statics.add(new JLabel(Ariane.lang.getString("AboutBox.statistics.size")+" "+ df.format(stat)+Ariane.lang.getString("AboutBox.statistics.megabytes")));
			panel.add(statics);

			statics = new JPanel(new FlowLayout(FlowLayout.LEFT));
			statics.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.lang.getString("AboutBox.statistics.usedPercent")));
			double percent = ((double)Runtime.getRuntime().totalMemory()-Runtime.getRuntime().freeMemory())/Runtime.getRuntime().maxMemory();
			statics.add(new JLabel(Ariane.lang.getString("AboutBox.statistics.size")+" "+df.format(percent)+"%"));
			panel.add(statics);

			statics = new JPanel(new FlowLayout(FlowLayout.LEFT));
			statics.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.lang.getString("AboutBox.statistics.availableProcessors")));
			statics.add(new JLabel(Ariane.lang.getString("AboutBox.statistics.number")+" "+Runtime.getRuntime().availableProcessors()));
			panel.add(statics);

			OperatingSystemMXBean operatingSystemMXBean = ManagementFactory.getOperatingSystemMXBean();

			if (operatingSystemMXBean instanceof UnixOperatingSystemMXBean) {
				UnixOperatingSystemMXBean operatingSystem = (UnixOperatingSystemMXBean )operatingSystemMXBean;
				statics= new JPanel(new FlowLayout(FlowLayout.LEFT));
				statics.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.lang.getString("AboutBox.statistics.maxFileDescriptorCount")));
				statics.add(new JLabel(Ariane.lang.getString("AboutBox.statistics.number")+" "+operatingSystem.getMaxFileDescriptorCount()));
				panel.add(statics);

				statics= new JPanel(new FlowLayout(FlowLayout.LEFT));
				statics.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.lang.getString("AboutBox.statistics.openFileDescriptorCount")));
				statics.add(new JLabel(Ariane.lang.getString("AboutBox.statistics.number")+" "+operatingSystem.getOpenFileDescriptorCount()));
				panel.add(statics);
			}

			tabbedPane.addTab(Ariane.lang.getString("AboutBox.statistics.title"), panel);
		}

		// Tab 5: Legend.
		{
			Box panel=Box.createVerticalBox();

			JPanel panel1 = new JPanel(new BorderLayout());
			panel.add( panel1);
			panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.lang.getString("AboutBox.legend.header")));
			String text = Ariane.lang.getString("AboutBox.legend.text");
			JEditorPane t = new JEditorPane("text/html",text);
			t.setEditable(false);
			panel1.add(t,BorderLayout.CENTER);
			tabbedPane.addTab(Ariane.lang.getString("AboutBox.legend.title"), panel);
		}
		setLocationRelativeTo(parent);
	}
}
