/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

package fr.greyc.pantheon.ariane.ui;

import fr.greyc.pantheon.ariane.Ariane;

import fr.greyc.pantheon.ariane.graph.item.GraphItem;
import fr.greyc.pantheon.ariane.graph.item.OperatorItem;
import fr.greyc.pantheon.ariane.graph.item.etc.ItemSignatureParser;
import fr.greyc.pantheon.ariane.utils.GDialog;
import fr.greyc.pantheon.ariane.utils.MutableTreeItem;
import fr.greyc.pantheon.ariane.utils.TreeItem;


import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.lang.reflect.MalformedParameterizedTypeException;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.tree.DefaultTreeModel;

/**
 * The class <code>OperatorEditor</code> is a dialog box
 * that lets the user edit operator properties.
 * 
 * @author Régis CLOUARD - Jul 24, 2010
 * @author Régis CLOUARD - Dec 27, 2011 (add default parameters)
 */
public final class OperatorEditor extends GDialog {
	/** The operator to describe. */
	private TreeItem _currentOperator;
	/** the node that support the operator. */
	private MutableTreeItem _currentNode;
	/** The tree model. */
	private DefaultTreeModel _treeModel;

	/**
	 * Creates and shows the dialog box for editing operators.
	 * 
	 * @param parent the parent for this dialog box.
	 * @param currentNode the node which supports the operator.
	 * @param treeModel the tree model.
	 */
	public OperatorEditor( JDialog parent, MutableTreeItem currentNode, DefaultTreeModel treeModel ) {
		super(parent, Ariane.lang.getString("OperatorEditor.title"), true);
		_currentNode=currentNode;
		_treeModel = treeModel;
		_currentOperator = (TreeItem)_currentNode.getUserObject();
		addWindowListener(new WindowAdapter() {
			@Override
			public final void windowClosing(WindowEvent e) {
				cancelAction();
			}
		});
		displayContent();
		setLocationRelativeTo(parent);
	}

	/** Editor of the operator name. */
	private JTextField textfield1;
	/** Editor of the binary. */
	private JTextField textfield2;
	/** Editor of the command line format. */
	private JTextField textfield2bis;
	/** Editor of the document URL. */
	private JTextField _docURLField;
	/** Editor of the default values. */
	private JTextField _signature;
	/** Editor of the number of parameters. */
	private JSpinner textFieldiP;
	/** Editor of the number of inputs. */
	private JSpinner textFieldiI;
	/** Editor of the number of outputs. */
	private JSpinner textFieldiO;

	/**
	 * Sets the content of the dialog box.
	 */
	private void displayContent( ) {
		JPanel textPanel = new JPanel();
		textPanel.setLayout(new GridBagLayout());
		textPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.lang.getString("OperatorEditor.borderTitle")));

		GridBagConstraints c = new GridBagConstraints();

		c.fill = GridBagConstraints.BOTH;
		c.anchor=GridBagConstraints.WEST;
		textPanel.add(new JLabel(Ariane.lang.getString("OperatorEditor.name")),c);
		textfield1 = new JTextField(30);
		textfield1.setText(_currentOperator.getName());

		c.gridwidth = GridBagConstraints.REMAINDER; //end row
		textPanel.add(textfield1,c);

		c.gridwidth = 1;
		textPanel.add(new JLabel(Ariane.lang.getString("OperatorEditor.binary")),c);
		textfield2 = new JTextField(30);
		String binary = GraphItem.getAbsoluteBinary(OperatorItem.getCommandPart(_currentOperator.getBinary()));
		if (binary !=null && binary.length()>0) {
			textfield2.setText(binary);
		}
		textPanel.add(textfield2,c);
		JButton button1 = new JButton(Ariane.lang.getString("OperatorEditor.load"));
		button1.addActionListener( new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				JFileChooser fileChooser = new JFileChooser();
				fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
				fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
				if (textfield2.getText().length()>0) {
					fileChooser.setSelectedFile(new File(textfield2.getText()));
				}
				if (fileChooser.showOpenDialog(OperatorEditor.this) == JFileChooser.APPROVE_OPTION) {
					textfield2.setText(fileChooser.getSelectedFile().getAbsolutePath());
					textfield2.setCaretPosition(0);
				}
			}
		});
		c.gridwidth = GridBagConstraints.REMAINDER; //end row
		textPanel.add(button1, c);

		c.gridwidth =1;
		textPanel.add(new JLabel(Ariane.lang.getString("OperatorEditor.synopsis")),c);
		textfield2bis = new JTextField(35);
		String format=OperatorItem.getFormatPart(_currentOperator.getBinary());
		textfield2bis.setText(format);
		c.gridwidth = GridBagConstraints.REMAINDER; //end row
		textPanel.add(textfield2bis,c);

		c.gridwidth = GridBagConstraints.REMAINDER; //end row
		textPanel.add(new JLabel("<html><i>"+Ariane.lang.getString("OperatorEditor.synopsisDescription")+"</i><html>"),c);

		c.gridwidth = 1;
		textPanel.add(new JLabel(Ariane.lang.getString("OperatorEditor.manual")),c);
		_docURLField = new JTextField(35);
		_docURLField.setText(_currentOperator.getDocURL());
		textPanel.add(_docURLField,c);
		JButton button2 = new JButton(Ariane.lang.getString("OperatorEditor.load"));
		button2.addActionListener( new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				JFileChooser fileChooser = new JFileChooser(".");
				fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
				fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
				if (_docURLField.getText().length()>6) {
					fileChooser.setSelectedFile(new File(_docURLField.getText().substring(7)));
				}
				if (fileChooser.showOpenDialog(OperatorEditor.this) == JFileChooser.APPROVE_OPTION) {
					_docURLField.setText("file://"+fileChooser.getSelectedFile().getAbsolutePath());
					_docURLField.setCaretPosition(0);
				}
			}
		});

		JButton button3 = new JButton(Ariane.lang.getString("OperatorEditor.edit"));
		button3.addActionListener( new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				HTMLDocumentEditor editor = new HTMLDocumentEditor(OperatorEditor.this);
				if (_docURLField.getText().length()>6) {
					editor.setSelectedFile(new File(_docURLField.getText().substring(7)));
				}
				if (editor.edit() == HTMLDocumentEditor.OK_OPTION && editor.getSelectedFile() != null ) {
					_docURLField.setText("file://"+editor.getSelectedFile().getAbsolutePath());
					_docURLField.setCaretPosition(0);
				}
			}
		});
		textPanel.add(button2, c);
		c.gridwidth = GridBagConstraints.REMAINDER; //end row
		textPanel.add(button3, c);

		c.gridwidth =1;
		c.fill = GridBagConstraints.NONE;

		textPanel.add(new JLabel(Ariane.lang.getString("OperatorEditor.parameters")),c);
		SpinnerModel spModel;
		spModel = new SpinnerNumberModel(_currentOperator.getParameters(),0,10,1);
		textFieldiP = new JSpinner(spModel);
		c.gridwidth = GridBagConstraints.REMAINDER; //end row
		textPanel.add(textFieldiP,c);

		c.gridwidth =1;
		textPanel.add(new JLabel(Ariane.lang.getString("OperatorEditor.signature")),c);
		_signature = new JTextField(30);
		String defaultValues=_currentOperator.getSignature();
		_signature.setText(defaultValues);
		c.gridwidth = GridBagConstraints.REMAINDER; //end row
		textPanel.add(_signature,c);

		c.gridwidth = GridBagConstraints.REMAINDER; //end row
		textPanel.add(new JLabel("<html><i>"+Ariane.lang.getString("OperatorEditor.signatureExample")+"</i></html>"),c);

		c.gridwidth =1;
		textPanel.add(new JLabel(Ariane.lang.getString("OperatorEditor.inputs")),c);
		spModel = new SpinnerNumberModel(_currentOperator.getInImages(),0,10,1);
		textFieldiI = new JSpinner(spModel);
		c.gridwidth = GridBagConstraints.REMAINDER; //end row
		textPanel.add(textFieldiI,c);

		c.gridwidth =1;
		textPanel.add(new JLabel(Ariane.lang.getString("OperatorEditor.outputs")),c);
		spModel = new SpinnerNumberModel(_currentOperator.getOutImages(),0,10,1);
		textFieldiO = new JSpinner(spModel);
		c.gridwidth = GridBagConstraints.REMAINDER; //end row
		textPanel.add(textFieldiO,c);

		// cancel button
		JButton cancelButton = new JButton();
		cancelButton.setAction(new AbstractAction(Ariane.lang.getString("OperatorEditor.cancel"),
				new ImageIcon(getClass().getClassLoader().getResource(Ariane.iconDir+"gtk-cancel.png"))) {
			@Override
			public final void actionPerformed(ActionEvent e) {
				cancelAction();
			}
		});

		JButton closeButton = new JButton(Ariane.lang.getString("OperatorEditor.validate"),
				new ImageIcon(getClass().getClassLoader().getResource(Ariane.iconDir+"gtk-apply.png")));
		JPanel buttonPanel = new JPanel();
		closeButton.addActionListener( new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				applyAction();
			}
		});
		buttonPanel.add(closeButton);
		buttonPanel.add(cancelButton);
		setDefaultButton(closeButton);
		setCancelButton(cancelButton);

		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(textPanel, BorderLayout.CENTER);
		getContentPane().add(buttonPanel, BorderLayout.SOUTH);
		pack();
	}

	/**
	 * Validates information.
	 */
	private void cancelAction( ) {
		setVisible(false);
		dispose();
	}

	/**
	 * Validates information.
	 */
	private void applyAction( ) {
		try {
			_currentOperator.setName(textfield1.getText());
			_currentOperator.setBinary(textfield2.getText()+OperatorItem.separator+textfield2bis.getText());
			_currentOperator.setDocURL(_docURLField.getText());
			_currentOperator.setParameters((Integer)textFieldiP.getValue());
			_currentOperator.setInImages((Integer)textFieldiI.getValue());
			_currentOperator.setOutImages((Integer)textFieldiO.getValue());
			_treeModel.nodeChanged(_currentNode);
			ItemSignatureParser.parse(_signature.getText(), _currentOperator.getParameters());
			_currentOperator.setSignature(_signature.getText());
		} catch( NumberFormatException e ) {
			e.printStackTrace();
			Ariane.workspace.printErrorMessage(e.getLocalizedMessage());
		} catch ( MalformedParameterizedTypeException e) {
//			e.printStackTrace();
			Ariane.workspace.printErrorMessage(Ariane.lang.getString("OperatorEditor.error1"));
		}
		setVisible(false);
		_currentOperator=null;
		dispose();
	}
}