/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

package fr.greyc.pantheon.ariane.utils;

import java.awt.Frame;
import java.awt.event.KeyEvent;

import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.KeyStroke;

/**
 * <code>GDialog</code> overrides the <code>JDialog</code> to 
 * add some specific facilities, such as default and cancel buttons.
 * 
 * @author Régis CLOUARD - Jun 28, 2010
 */
public class GDialog extends JDialog {
	/**
	 * Creates a dialog with the specified title, owner Frame  and modality.
	 * 
	 * @param owner the Frame from which the dialog is displayed.
	 * @param title the String to display in the dialog's title bar.
	 * @param modal specifies whether dialog blocks user input or not.
	 */
	public GDialog( Frame owner, String title, boolean modal ) {
		super(owner,title, modal);
	}

	/**
	 * Creates a dialog with the specified title, owner Frame  and modality.
	 * 
	 * @param owner the parent dialog from which the dialog is displayed.
	 * @param title the String to display in the dialog's title bar.
	 * @param modal specifies whether dialog blocks user input or not.
	 */
	public GDialog( JDialog owner, String title, boolean modal ) {
		super(owner,title, modal);
	}

	/**
	 * Sets the button that corresponds to the escape key.
	 * 
	 * @param button the JButton which is to be the cancel button. 
	 */
	public void setCancelButton( JButton button ) {
		InputMap inputMap = getRootPane().getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		ActionMap actionMap = getRootPane().getActionMap();
		KeyStroke cancelKeyStroke = KeyStroke.getKeyStroke((char)KeyEvent.VK_ESCAPE,0);
		inputMap.put(cancelKeyStroke, "cancel");
		actionMap.put("cancel", button.getAction());
	}

	/**
	 * Sets the defaultButton property, which determines the current default
	 * button for this Jdialog.
	 * 
	 * @param button the JButton which is to be the default button. 
	 */
	public void setDefaultButton( JButton button ) {
		getRootPane().setDefaultButton(button);
	}
}
