/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c) 2013, GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Pandore
 */

package fr.greyc.pantheon.ariane.utils;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;

import javax.swing.border.AbstractBorder;

/**
 * A rounded border.
 * 
 * @author Régis CLOUARD - Mar 28, 2013
 */
public final class RoundedBorder extends AbstractBorder {
	public static final int RAISED  = 0;
	public static final int LOWERED = 1;

	private int _etchType;
	private Color _highlight;
	private Color _shadow;

	public RoundedBorder( int etchType ) {
		_etchType = etchType;
	}

	public RoundedBorder( int etchType, Color highlight, Color shadow ) {
		_etchType = etchType;
		_highlight = highlight;
		_shadow = shadow;
	}

	@Override
	public final void paintBorder( Component c, Graphics g, int x, int y, int width, int height ) {
		Graphics2D graphics = (Graphics2D) g;
		g.translate(x, y);
		g.setColor(_etchType == LOWERED? getShadowColor(c) : getHighlightColor(c));
		graphics.setStroke(new BasicStroke(2));
		final int shadowGap = 2;
		graphics.drawRoundRect(0, 0, width - shadowGap, height - shadowGap, RoundedPanel.cornerArc.width, RoundedPanel.cornerArc.height);

		g.setColor(_etchType == LOWERED? getHighlightColor(c) : getShadowColor(c));
		graphics.setStroke(new BasicStroke(1));
		g.translate(x+1, y+1);
//		shadowGap = 2;
		graphics.drawRoundRect(0, 0, width - shadowGap, height - shadowGap, RoundedPanel.cornerArc.width, RoundedPanel.cornerArc.height);
		g.translate(-x-1, -y-1);
	}

	@Override
	public final Insets getBorderInsets( Component c ) {
		return new Insets(2, 2, 3, 3);
	}

	@Override
	public final Insets getBorderInsets( Component c, Insets insets ) {
		insets.top = 2;
		insets.left = 2;
		insets.bottom = 3;
		insets.right = 3;
		return insets;
	}

	@Override
	public final boolean isBorderOpaque() { return true; }

	private final Color getShadowColor( Component c ) {
		return _shadow != null? _shadow : c.getBackground().darker();
	}

	private final Color getHighlightColor( Component c ) {
		return _highlight != null? _highlight : c.getBackground().brighter();
	}
}